/******************************************************************
  Non-Clausal DPLL Solver
  Version: 2.5

  Christian Thiffault
  University of Toronto
  November 1st 2003 (April 2004)

  *****************************************************************

   MemManager.h

  This file contains the C++ definition of a memory manager.  Large
  blocks of memory are allocated as needed and the user calls
  ::calloc to allocate smaller arrays as needed.  This guarantees
  that the small blocks will be next to each other.

  Currently, this file is mainly used to manage NoGoods and NoGood
  watches (linked list).

  Note: Still a little messy because of cleanup code...

  *****************************************************************/

#ifndef __MEMMANAGER_H__
#define __MEMMANAGER_H__




class MemManager {

protected:

  int *first_block;            // Ptr to the first block allocated
  int *current_block;          // Ptr to the current block
  int size;                    // Size of each block



public:

 // Constructor - allocates the first block
  MemManager(int s) {
    size = s;
    first_block = createNewBlock();
    current_block = first_block;
  }


 // Destructor - frees all the blocks allocated
  ~MemManager() {
    int *next_block = first_block;
    while (next_block != NULL) {
      current_block = next_block;
      next_block = (int*) next_block[0];
      delete[] current_block;
    }
  }



 // Returns a pointer to the start of the next allocation inside
 // the larger block of memory
  void inline *calloc(int num, int s) {

    int loc;
    num *= s;
    num = (num / sizeof(int)) + ((num % sizeof(int))?1:0);

    while (num > size - current_block[1] + 3) {
      assert(num < size);
      if ((int*)current_block[0] == NULL)
        current_block[0] = (int) createNewBlock();
      current_block = (int*) current_block[0];
      current_block[1] = 3;
      current_block[2] = 0;
    }
    loc = current_block[1];
    current_block[1] += num;
    //printf("Calloc returning %p\n",(void*) &(current_block[loc]));
    return (void*) &(current_block[loc]);
  }



 // Look whether an address is within any of the blocks allocated
 // by this memory manager
  bool inline member(int *address) {
    int *current = first_block;
    while (current != NULL) {
      if ((address > current) && (address < current + size))
        return true;
      current = (int*) current[0];
    }
    return false;
  }



 // Get the requested block as well as the start and the end of this
 // block
  int inline *getBlock(int num, int **start, int **end) {
    int *current = first_block;
    for (int i=0; i<num; i++) {
      current = (int*) current[0];
      if (current == NULL)
        return NULL;
    }
    *start = &(current[2]);
    *end = &(current[current[1]]);
    return current;
  }


 // Get the number of allocated blocks
  int inline getNumBlocks() {
    int *current = first_block;
    int num = 1;
    while (current != current_block) {
      current = (int*) current[0];
      num++;
    }
    return num;
  }


 // Get a pointer to the last possible entry of a block
  int inline *getLastBlockEntry(int num) {
    int *start, *end;
    int *current = getBlock(num, &start, &end);
    return &current[size+2];
  }


 // Set the next empty location of a given block
  void inline setNext(int block, int *empty) {
    int *start, *end;
    current_block = getBlock(block, &start, &end);
    assert(current_block != NULL);
    if (empty == NULL) {
      current_block[1] = 3;
      current_block[2] = 0;
    } else {
      assert(*(empty-1) == 0);
      assert((empty - start) <= size);
      current_block[1] = (empty - start) + 2;
    }
    assert(current_block[current_block[1]-1] == 0);
  }


 // Free all the memory in the memmanager (placing empty pointers
 // to the start of the memory blocks)
  void inline free() {
    current_block = first_block;
    while (current_block != NULL) {
      current_block[1] = 3;
      current_block[2] = 0;
      current_block = (int*) current_block[0];
    }
    current_block = first_block;
  }


 // Output the information inside the block for debugging
  void printBlock() {
    int *current = first_block;
    while (current != NULL) {
      printf("\nBLOCK: ");
      for (int i=0; i<current[1]; i++) {
        printf("%d ", current[i]);
      }
      printf("\n\n");
      current = (int*) current[0];
    }
  }



protected:

 // Allocate memory for a new block and initialize the header
  int inline *createNewBlock() {
    int *block = new int[size+3];
    int *null_ptr = NULL;  // Gets around a warning

    //printf("CreateNewBlock gives 0x%08x\n",(int) block);
    assert(block);
    block[0] = (int) null_ptr;
    block[1] = 3;
    block[2] = 0;
    return block;
  }

};




#endif

