/******************************************************************
  Non-Clausal DPLL Solver
  Version: 2.5

  Christian Thiffault
  University of Toronto
  July 30th 2003 (April 2004)

  *****************************************************************

  Solver.h

  Contains the external prototypes for the solving module.  This
  module takes a tree as a compact representation (contained in the
  global pointer g_nodes) and attempts to find an assignment
  that satisfies the expression.

  *****************************************************************/

#ifndef __SOLVER_H__
#define __SOLVER_H__

#include "noclause.h"
#include "uniquestack.h"




/************************** Constants *****************************/

 // Return values used within the solver files
 // (solver, propagate, undo and nogood)
#define SOLVER_UNSATISFIABLE            0
#define SOLVER_SATISFIABLE              1
#define SOLVER_OK                       2
#define SOLVER_CONFLICT                 3

 // VSIDS constants
#define SOLVER_VDS_DECAY_RATE      0x0100
#define SOLVER_VDS_DECAY_FACTOR         2
#define SOLVER_VDS_INITIAL_STEP    0x0100

 // Restart constants
#define SOLVER_RESTART_INITIAL       5000
#define SOLVER_RESTART_STEP          1500
#define SOLVER_RESTART_STEP_INCR      200




/******************* Structure Declarations ***********************/

/*
   Record containing the index of a node as well as its score.
   This structure is used in an array of nodes indexed by their
   score.
    - index of the node
    - score of the node (max of true_score and false_score)
 */
typedef struct node_ordering {
  int node;
  int score;
} node_ordering;







/*************************** Macros *******************************/
/********************** Global Variables **************************/

extern int g_conflict_node;
extern reasoninfo g_conflict_reason;



/*********************** Inline Functions *************************/

 // Get the true, false and overall VSIDS scores for each node
int inline Solver_GetTrueScore(int node) {
  return g_nodes.score[(node<<1)+1];
}
int inline Solver_GetFalseScore(int node) {
  return g_nodes.score[node<<1];
}
int inline Solver_GetScore(int node) {
  return (g_nodes.score[node<<1] > g_nodes.score[(node<<1)+1])?
          g_nodes.score[node<<1]:g_nodes.score[(node<<1)+1];
}


 // Functions to set and increment the various scores of a node
void inline Solver_SetTrueScore(int node, int score) {
  g_nodes.score[(node<<1)+1] = score;
}
void inline Solver_SetFalseScore(int node, int score) {
  g_nodes.score[node<<1] = score;
}
void inline Solver_IncrementTrueScore(int node) {
  g_nodes.score[(node<<1)+1]++;
}
void inline Solver_IncrementFalseScore(int node) {
  g_nodes.score[node<<1]++;
}








/****************** External Function Declarations ****************/

extern int  Solver_SolveDAG();
extern int  Solver_Assign(int node, byte value, reasoninfo reason);
extern void Solver_UpdateVariableScores(int *clause, int size);
extern void Solver_DecayVariableScores(int factor);


#endif

