/******************************************************************
  Non-Clausal DPLL Solver
  Version: 2.5

  Christian Thiffault
  University of Toronto
  November 10th 2003 (April 2004)

  *****************************************************************

   Vector.h

  Class definition/description of a dynamically growing array.
  The array of integers doubles in size once it reaches the
  current capacity.

  *****************************************************************/
  
#ifndef __VECTOR_H__
#define __VECTOR_H__

#include "memmanager.h"



/************************** Constants *****************************/

// This constant is the recommended initial size
#define VECTOR_INITIAL_SIZE   50




/******************* Structure Declarations ***********************/

class Vector {

protected:

  int size;
  int capacity;
  int *set;
  MemManager *mem_mngr; // Contiguous block of memory from which the
                        // vector should be allocated


public:

 // Constructor
  Vector(MemManager *mngr, int s) {
    mem_mngr = mngr;
    capacity = s;
    size = 0;
    set = (int*) mem_mngr->calloc (capacity, sizeof(int));
    assert(set);
  }



 // Add an element to the array, reallocating the array if necessary
  int inline add(int info) {
    if (size >= capacity) {
      int *new_set = (int*) mem_mngr->calloc (capacity << 1, sizeof(int));
      memcpy(new_set, set, capacity * sizeof(int));
      set = new_set;
      capacity <<= 1;
    }
    set[size] = info;
    return size++;
  }



 // Remove an element at location 'loc', swapping the last element
 // to the removed location and decrementing the size of the array
  void inline remove(int loc, int *info) {
    set[loc] = set[--size];
    if (size == 0) {
      *info = -1;
    } else {
      *info = set[loc];
    }
  }



 // Empty the vector and reallocate the memory
 //  - this function is used during cleanup to compact the database
  void inline realloc() {
    size = 0;
    capacity = (capacity+1) >> 1;
    set = (int*) mem_mngr->calloc (capacity, sizeof(int));
  }



  int inline getSize() { return size; }
  int inline getElementAt(int loc) { return set[loc]; }
  void inline setElementAt(int loc, int e) { set[loc] = e; }

};







#endif

